import requests
from .auth import Auth
from .grade import Grade
from .absence import Absence
from .time_schedule import TimeSchedule
from .credit import Credit


class Esprit:
    def __init__(self):
        self.session = requests.Session()
        self.auth = Auth()
        self.grade_scrape = Grade(self.session)
        self.grade_scrape = Grade(self.session)
        self.absence_scrape = Absence(self.session)
        self.time_schedule_scrape = TimeSchedule(self.session)
        self.credit = Credit(self.session)

    def login(self, username, password):
        cookies = self.auth.login(username, password)
        self.session.cookies.update(cookies)

    def get_grades(self):
        return self.grade_scrape.get_grades()

    def get_absences(self):
        return self.absence_scrape.get_absences()

    def get_table_schedules(self):
        return self.time_schedule_scrape.get_table_schedules()

    def get_last_week_schedule(self):
        return self.time_schedule_scrape.get_last_week_schedule()

    def download_files(self, schedule, download_path):
        return self.time_schedule_scrape.download_files(schedule, download_path)

    def get_class_week_schedule(self, file_path, class_name, result_path):
        return self.time_schedule_scrape.get_class_week_schedule(file_path, class_name, result_path)

    def get_credits(self):
        return self.credit.get_credits()



import requests
from bs4 import BeautifulSoup


class Credit:
    """
    A class used to represent a Credit.

    ...

    Attributes
    ----------
    url : str
        a formatted string that represents the URL of the credit page
    session : requests.Session
        a Session object from the requests library

    Methods
    -------
    get_credits():
        Returns a list of credits for the student.
    """

    def __init__(self, session):
        self.url = "https://esprit-tn.com/ESPOnline/Etudiants/Historique_Cr%C3%A9dit.aspx"
        self.session = session

    def get_credits(self):
        """
        Returns a list of credits for the student.

        Returns
        -------
        list
            a list of credits, each represented as a list of strings. The first list is the headers.
            Returns None if the page does not contain the expected text.
        """
        response = self.session.get(self.url)
        soup = BeautifulSoup(response.text, 'html.parser')

        # Check if the <span> tag with the class "style5" and the text "Historique des Crédits" exists
        span_tag = soup.find(
            'span', {'class': 'style5'}, text='   Historique des Crédits  ')
        if span_tag is None:
            print("The page does not contain the expected text.")
            return None

        table = soup.find('table', {'id': 'ContentPlaceHolder1_GridView1'})
        rows = table.find_all('tr')
        headers = [cell.text.strip() for cell in rows[0].find_all('th')]
        grades = [headers] + [[cell.text.strip() for cell in row.find_all('td')]
                              for row in rows[1:]]  # Skip header row
        return grades



import requests
from bs4 import BeautifulSoup


class Absence:
    """
    A class used to represent an Absence.

    ...

    Attributes
    ----------
    url : str
        a formatted string that represents the URL of the absence page
    session : requests.Session
        a Session object from the requests library

    Methods
    -------
    get_absences():
        Returns a list of absences for the student.
    """

    def __init__(self, session):
        self.url = "https://esprit-tn.com/ESPOnline/Etudiants/absenceetud.aspx"
        self.session = session

    def get_absences(self):
        """
        Returns a list of absences for the student.

        Returns
        -------
        list
            a list of absences, each represented as a list of strings. The first list is the headers.
            Returns None if the page does not contain the expected text.
        """

        response = self.session.get(self.url)
        soup = BeautifulSoup(response.text, 'html.parser')

        # Check if the <strong> tag with the text "Absence" exists
        strong_tag = soup.find('strong', text='Absence')
        if strong_tag is None:
            print("The page does not contain the expected text.")
            return None

        table = soup.find('table', {'id': 'ContentPlaceHolder1_GridView2'})
        rows = table.find_all('tr')
        headers = [cell.text.strip() for cell in rows[0].find_all('th')]
        absences = [headers] + [[cell.text.strip() for cell in row.find_all('td')]
                                for row in rows[1:]]  # Skip header row
        return absences



from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.chrome.options import Options


class Auth:
    def __init__(self, driver_path=None, driver=None, debug=False, headless=True):
        self.login_url = "https://esprit-tn.com/esponline/online/default.aspx"
        self.debug = debug

        # Set up Chrome options
        chrome_options = Options()
        if headless:
            chrome_options.add_argument("--headless")

        self.driver = driver if driver else webdriver.Chrome(
            executable_path=driver_path, options=chrome_options)

    def login(self, id, password):
        self.driver.get(self.login_url)

        # Fill in the ID
        id_input = WebDriverWait(self.driver, 20).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="ContentPlaceHolder1_TextBox3"]'))
        )
        id_input.send_keys(id)

        # Click the 'Suivant' button using XPath
        suivant_button = WebDriverWait(self.driver, 20).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="ContentPlaceHolder1_Button3"]'))
        )
        suivant_button.click()

        # Fill in the password using XPath
        password_input = WebDriverWait(self.driver, 20).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="ContentPlaceHolder1_TextBox7"]'))
        )
        password_input.send_keys(password)

        # Click the 'Connexion' button using XPath
        connexion_button = WebDriverWait(self.driver, 20).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="ContentPlaceHolder1_ButtonEtudiant"]'))
        )
        connexion_button.click()

        # Check if login was successful
        if 'Vous pouvez consulter dans cet espace :' in self.driver.page_source:
            print('''
                  -----------------
                  Login successful!
                  -----------------
                  ''')
            return self.driver.get_cookies()
        else:
            print('''
                  -------------
                  Login failed!
                  -------------
                  ''')


# Code for debugging login
if __name__ == "__main__":
    auth = Auth(
        driver_path="C:/custom/bins/chromedriver-win64/chromedriver.exe", debug=True)
    id = "your_id"
    password = "your_password"
    session = auth.login(id, password)



import requests
from bs4 import BeautifulSoup
from datetime import datetime
import re
import os
from PyPDF2 import PdfReader, PdfWriter


class TimeSchedule:
    """
    A class used to represent a Time Schedule.

    ...

    Attributes
    ----------
    url : str
        a formatted string that represents the URL of the time schedule page
    session : requests.Session
        a Session object from the requests library

    Methods
    -------
    get_table_schedules():
        Returns a list of time schedules for the student.
    get_last_week_schedule():
        Returns the most recent weekly schedule.
    download_files(schedule: list):
        Downloads the files associated with a given schedule.
    get_class_week_schedule(file_path: str, class_name: str):
        Extracts the weekly schedule for a specific class from a given file.
    """

    def __init__(self, session):
        self.url = "https://esprit-tn.com/ESPOnline/Etudiants/Emplois.aspx"
        self.session = session

    def get_table_schedules(self):
        """
        Returns a list of time schedules for the student.

        Returns
        -------
        list
            a list of time schedules, each represented as a list of strings.
            Returns None if the page does not contain the expected text.
        """
        response = self.session.get(self.url)
        soup = BeautifulSoup(response.text, 'html.parser')

        # Check if the <strong> tag with the text "Emploi du temps" exists
        strong_tag = soup.find('strong', text='Emploi du temps')
        if strong_tag is None:
            print("The page does not contain the expected text.")
            return None

        table = soup.find('table', {'id': 'ContentPlaceHolder1_GridView1'})
        rows = table.find_all('tr')
        time_schedules = []
        for row in rows[1:]:
            row_data = []
            for cell in row.find_all('td'):
                cell_text = cell.text.strip()
                cell_link = cell.find('a')
                if cell_link is not None:
                    cell_text += ' ' + cell_link.get('href')
                row_data.append(cell_text)
            time_schedules.append(row_data)
        return time_schedules

    def get_last_week_schedule(self):
        """
        Returns the most recent weekly schedule.

        Returns
        -------
        list
            the most recent weekly schedule, represented as a list of strings.
            Returns None if no schedules are found.
        """
        time_schedules = self.get_table_schedules()
        if time_schedules is None:
            return None

        # Filter schedules that contain "Semaine" and extract dates
        week_schedules = [
            schedule for schedule in time_schedules if "Semaine" in schedule[0]]
        date_format = "%d-%m-%Y"
        dates_and_schedules = []

        for schedule in week_schedules:
            # Extract date from file name
            match = re.search(r"\d{2}-\d{2}-\d{4}", schedule[0])
            if match:
                date_str = match.group()
                date = datetime.strptime(date_str, date_format)
                dates_and_schedules.append((date, schedule))

        # Sort by date and return the latest schedule
        dates_and_schedules.sort(key=lambda x: x[0])
        return dates_and_schedules[-1][1] if dates_and_schedules else None

    def download_files(self, schedule, download_path):
        """
        Downloads the files associated with a given schedule.

        Parameters
        ----------
            schedule : list
                the schedule to download files for, represented as a list of strings
            download_path : str
                the path where the file should be downloaded

        Returns
        -------
        str
            the path to the downloaded file
        """
        response = self.session.get(self.url)
        soup = BeautifulSoup(response.text, 'html.parser')

        # Extract __VIEWSTATE and __EVENTVALIDATION values
        viewstate = soup.find('input', {'id': '__VIEWSTATE'})['value']
        eventvalidation = soup.find(
            'input', {'id': '__EVENTVALIDATION'})['value']

        # Extract eventTarget from the JavaScript function call
        event_target = schedule[1].split("'")[1]

        # Send a POST request to mimic the postback
        post_data = {
            '__EVENTTARGET': event_target,
            '__EVENTARGUMENT': '',
            '__VIEWSTATE': viewstate,
            '__EVENTVALIDATION': eventvalidation,
        }
        file_response = self.session.post(self.url, data=post_data)

        # Save the file
        file_path = os.path.join(download_path, schedule[0])
        with open(file_path, 'wb') as f:
            f.write(file_response.content)

        return file_path

    def get_class_week_schedule(self, file_path, class_name, result_path):
        """
        Extracts the weekly schedule for a specific class from a given file.

        Parameters
        ----------
            file_path : str
                the path to the file to extract the schedule from
            class_name : str
                the name of the class to extract the schedule for
            result_path : str
                the path where the result PDF will be saved

        Returns
        -------
        str
            the path to the extracted schedule, or None if the class is not found in the file
        """
        # Open the existing PDF
        with open(file_path, "rb") as file:
            reader = PdfReader(file)

            # Assume class_name is found on a specific page
            for page_number in range(len(reader.pages)):
                page = reader.pages[page_number]
                content = page.extract_text()

                # If class_name is found in content
                if class_name in content:
                    writer = PdfWriter()
                    writer.add_page(page)

                    # Save the page as a new PDF
                    new_file_path = os.path.join(
                        result_path, f"{class_name}.pdf")
                    with open(new_file_path, "wb") as output_pdf:
                        writer.write(output_pdf)

                    return new_file_path

        return None



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# Read requirements.txt
with open(os.path.join(here, 'requirements.txt')) as f:
    requirements = f.read().splitlines()

VERSION = '0.3.1'
DESCRIPTION = ' A Python library for interacting with data from esprit-tn.com'
LONG_DESCRIPTION = 'esprit-py, provides a set of tools for interacting with data from the Esprit website. It includes functionalities for scraping grades, absences, time schedules, and credits. It also provides the ability to download files and get class week schedules.'

# Setting up
setup(
    name="esprit-py",
    version=VERSION,
    author="Lime1 (Aymen Hmani)",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=requirements,
    keywords=['python', 'api'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    package_data={"": ["requirements.txt"]},
    url="https://github.com/TheLime1/esprit.py"
)

'''
python setup.py sdist bdist_wheel
twine check dist/*
twine upload dist/*
-
username: __token__
password: pypi-...
'''
